﻿using System.Collections.ObjectModel;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.Wpf;


namespace _9408
{
    class RulerTrackInteractiveOverlay : TrackInteractiveOverlay
    {
        private const string currentFeatureKey = "CurrentFeature";
        private LineShape rulerLineShape;        
        private bool isTrack;

        public RulerTrackInteractiveOverlay()
            : base()
        {
            isTrack = false;
            this.TrackMode = TrackMode.Point;

            TrackShapeLayer.Open();
            TrackShapeLayer.Columns.Add(new FeatureSourceColumn("length"));
            //Sets the appearance of the ruler
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.FromArgb(150, GeoColor.StandardColors.DarkRed), 4, false);
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("length", new GeoFont("Arial", 10, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColor.SimpleColors.Black));
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.TextLineSegmentRatio = 100;
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.YOffsetInPixel = 10;
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        }

        protected override InteractiveResult MouseClickCore(InteractionArguments interactionArguments)
        {
            InteractiveResult interactiveResult = new InteractiveResult();

            if (!isTrack)
            {
                //Sets the first and last point of the line where the user clicks.                
                interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
                interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;

                rulerLineShape = new LineShape(new Collection<Vertex>() { new Vertex(interactionArguments.WorldX, interactionArguments.WorldY), new Vertex(interactionArguments.WorldX, interactionArguments.WorldY) });

                Feature newFeature = new Feature(rulerLineShape);
                if (TrackShapeLayer.InternalFeatures.Contains(currentFeatureKey)) TrackShapeLayer.InternalFeatures[currentFeatureKey] = newFeature;
                else TrackShapeLayer.InternalFeatures.Add(currentFeatureKey, newFeature);
            }
            else
            {   //Removes the line of the ruler at finishing dragging (at mouse up event).                
                interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
                interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;

                if (rulerLineShape != null)
                {
                    TrackShapeLayer.InternalFeatures.Remove(currentFeatureKey);
                    rulerLineShape = null;
                }
            }

            isTrack = !isTrack;

            return base.MouseClickCore(interactionArguments);
        }

        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            //Updates the line with the last point to where the mouse pointer is being dragged.
            InteractiveResult interactiveResult = new InteractiveResult();

            int PointCount = TrackShapeLayer.InternalFeatures.Count;
            if (TrackShapeLayer.InternalFeatures.Contains(currentFeatureKey)) PointCount--;

            if (rulerLineShape != null && PointCount > 0)
            {
                rulerLineShape.Vertices[rulerLineShape.Vertices.Count - 1] = new Vertex(interactionArguments.WorldX, interactionArguments.WorldY);
                interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
                interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;
            }
            return interactiveResult;
        }

        protected override void DrawCore(RectangleShape targetExtent, OverlayRefreshType overlayRefreshType)
        {
            if (rulerLineShape != null)
            {
                Feature feature = new Feature(rulerLineShape);
                double length = rulerLineShape.GetLength(GeographyUnit.DecimalDegree, DistanceUnit.Feet);
                feature.ColumnValues.Add("length", ((int)length).ToString() + " feet");
                if (TrackShapeLayer.InternalFeatures.Contains(currentFeatureKey)) TrackShapeLayer.InternalFeatures[currentFeatureKey] = feature;
                else TrackShapeLayer.InternalFeatures.Add(currentFeatureKey, feature);
            }

            base.DrawCore(targetExtent, overlayRefreshType);
        }
    }
}
